/*****************************************************************************
 *		main.c  --  modeld                                           *
 *                                                                           *
 *                                                                           *
 * (c) 2002 by Josef Kufner <jkufner@seznam.cz>                              *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 ****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/io.h>

#define DEBUG 	0
#include "debug.h"

#include "port.h"
#define USE_PORT	SERIAL_PORT	/* SERIAL_PORT nebo PARALLEL_PORT */

#define FIFO_OUT_NAME	"/tmp/modeld.output.fifo"
#define FIFO_IN_NAME	"/tmp/modeld.input.fifo"	/* neni pouzito */
#define LOCK_NAME	"/var/run/LCK.modeld"

int done = 0, sub_done = 0;

void on_signal(int sig) {
	if (sig == SIGPIPE) {
		sub_done = 1;
	} else {
		sub_done = 1;
		done = 1;
	}
}

int main(int argc, char *argv[]) {
	int fifo_out_fd, lock_fd;
	unsigned char byte;
	
	/* nastaveni signalu */
	signal(SIGINT, on_signal);
	signal(SIGTERM, on_signal);
	signal(SIGQUIT, on_signal);
	signal(SIGPIPE, SIG_IGN);

	/* Aktivace/Overeni zamku */
	if ((lock_fd = open(LOCK_NAME, O_RDWR | O_CREAT | O_EXCL, 0444))
			== -1)
	{
		if (errno == EEXIST) {
			fprintf(stderr, "modeld(%c): "
					"Tento program je ji sputn!\n",
					USE_PORT);
		} else {
			fprintf(stderr, "modeld(%c): ioperm error (%d): %s\n",
					USE_PORT, errno, strerror(errno));
		}
		exit(-1);
	}	
	
	/* Ziskani prav */
#if USE_PORT == SERIAL_PORT
	/* pro seriovy port */
	if (ioperm(SERIAL_BA, SERIAL_SIZE, 1)) {
		fprintf(stderr, "modeld(%c): ioperm error (%d): %s\n",
				USE_PORT, errno, strerror(errno));
		exit(-1);
	}
#elif USE_PORT == PARALLEL_PORT
	/* pro paralelni port */
	if (ioperm(PARALLEL_BA, PARALLEL_SIZE, 1)) {
		fprintf(stderr, "modeld(%c): ioperm error (%d): %s\n",
				USE_PORT, errno, strerror(errno));
		exit(-1);
	}
#endif
	
	while(!done) {
		/* tvorba fifo */
		umask(~0777);
		mkfifo(FIFO_OUT_NAME, 0666);

		/* Otevreni fifo pro cteni */
		if ((fifo_out_fd = open(FIFO_OUT_NAME, O_RDONLY | O_SYNC))
				== -1)
		{
			fprintf(stderr, "modeld(%c): open error (%d): %s\n",
					USE_PORT, errno, strerror(errno));
			exit(-1);
		}
	
		/* Prenos dat */
		byte = 0;
		sub_done = 0;
		while(!sub_done) {
			/* Nacteni 1B dat */
			if(read(fifo_out_fd, &byte, 1) > 0) {
				/* Poslani 1B dat na port */
#if USE_PORT == SERIAL_PORT
				/* pro seriovy port */
				outb(byte & SERIAL_CTRL_MASK,
						SERIAL_BA + SERIAL_CTRL_OFF);
				DBG_MSG(3, "0x%2X\n", byte & SERIAL_CTRL_MASK);
#elif USE_PORT == PARALLEL_PORT
				/* pro paralelni port */
				outb(PARALLEL_BA + PARALLEL_DATA_OFF,
						byte & PARALLEL_MASK);
				DBG_MSG(3, "0x%2X\n", byte & PARALLEL_DATA_MASK);
#else
				/* pro zadny port */
				printf("0x%2X\n", byte);
#endif
			}
		}

		/* Likvidace roury */
		close(fifo_out_fd);
		remove(FIFO_OUT_NAME);
	}

	/* Vraceni prav */
#if USE_PORT == SERIAL_PORT
	/* pro seriovy port */
	if (ioperm(SERIAL_BA, SERIAL_SIZE, 0)) {
		fprintf(stderr, "modeld(%c): ioperm error (%d): %s\n",
				USE_PORT, errno, strerror(errno));
		exit(-1);
	}
#elif USE_PORT == PARALLEL_PORT
	/* pro paralelni port */
	if (ioperm(PARALLEL_BA, PARALLEL_SIZE, 0)) {
		fprintf(stderr, "modeld(%c): ioperm error (%d): %s\n",
				USE_PORT, errno, strerror(errno));
		exit(-1);
	}
#endif

	/* Likvidace zamku */
	unlink(LOCK_NAME);
	
	/* Konec */	
	return 0;
}

